/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.CollectDeBlocks
 *
 * de.bsvrz.kex.tls.osi7.CollectDeBlocks is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.CollectDeBlocks is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.CollectDeBlocks.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.CollectDeBlocks.
 *
 *  de.bsvrz.kex.tls.osi7.CollectDeBlocks ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.CollectDeBlocks wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7;


import de.bsvrz.sys.funclib.debug.Debug;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * Ermglicht den Zugriff auf die FG/ID Kombinationen, fr die DE-Blcke nicht direkt an die Streckenstation versandt
 * werden, sondern erst nach Triggerung. Damit lassen sich z. B. die DE der FG 4 (Anzeigen) fr einen Anzeigequerschnitt
 * (physikalisch die DE der FG 4 an einem EAK) in einem Einzeltelegramm mit mehreren DE-Blcken fr diesen AQ versenden.
 * Die Triggerung erfolgt entweder fr ein Steuermodul (erzeugt dann jeweils Trigger fr die angeschlossenen EAK) oder
 * ein einzelnes EAK.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */

public class CollectDeBlocks {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Sonstige ---------------------------------------------------------------
	/**
	 * DebugLogger fr Debug-Ausgaben
	 */
	private static final Debug debug = Debug.getLogger();

	private static CollectDeBlocks collectDeBlocks;
// Private ----------------------------------------------------------------
	private int[][] _collectDeBlocks;

// ------------------------------------------------------------------------
// STATISCHE METHODEN
// ------------------------------------------------------------------------

	public synchronized static CollectDeBlocks getInstance() {
		if (collectDeBlocks == null) {
			collectDeBlocks = new CollectDeBlocks();
		}
		return collectDeBlocks;
	}

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Construktor, tut nichts. Singelton muss ber Aufruf von {@link #getInstance().#init(String)} initialisiert werden.
	 */
	private CollectDeBlocks() {

	}

// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * Initialisiert die Klasse, indem die bergebene Klasse dynamische geladen wird.
	 *
	 * @param collectDeBlockClassName Klassenname, der die Liste mit den zu puffernden FG/Typ/ID Kombinationen enthlt, fr
	 *                                die die DE-Blcke erst nach Triggerung zu Einzeltelegrammen zusammen gestellt
	 *                                werden.
	 */
	public void init(String collectDeBlockClassName) {
		Class classCollectDeBlocks;
		try {
			classCollectDeBlocks = Class.forName(collectDeBlockClassName);
			/**
			 * Methode im UmsetzungsModul, welche dynamisch zur Laufzeit ermittelt wird.
			 */
			Method method = classCollectDeBlocks.getMethod("getCollectDeBlocks", new Class[]{});
			_collectDeBlocks = (int[][]) method.invoke(null, new Object[]{});
		}
		catch (ClassNotFoundException e) {
			debug.error("Klasse zur Auflsung externer Objektreferenzen konnte nicht geladen werden: " + collectDeBlockClassName, e);
		}
		catch (NoSuchMethodException e) {
			debug.error("Methode 'getSourceDataReferences()' in der Klasse '" + collectDeBlockClassName + "' existiert nicht.", e);
		}
		catch (IllegalAccessException e) {
			debug.error("Fehler beim Zugriff auf die Methode 'getSourceDataReferences()' in der Klasse '" + collectDeBlockClassName + "'.", e);
		}
		catch (InvocationTargetException e) {
			debug.error("InvocationTargetException:", e);
		}
	}

	/**
	 * Prft, ob der DeBlock einer Fg/Typ/Id Kombination entspricht, fr die der Versand erst nach Triggerung am EAK oder
	 * SM gemeinsam mit anderen De-Blcken in einem Einzeltelegramm erfolgen soll (z. B. AQ-Schaltungen)
	 *
	 * @param deBlock
	 *
	 * @return true, wenn DeBlock gepuffert werden soll, false, falls direkter Versand erfolgen soll.
	 */
	public boolean isDeBlockToBuffer(DeBlock deBlock) {
		for (int i = 0; i < _collectDeBlocks.length; i++) {
			if (
					(_collectDeBlocks[i][0] == deBlock.getFg()) &&
					(_collectDeBlocks[i][1] == deBlock.getTyp()) &&
					(_collectDeBlocks[i][2] == deBlock.getId())
					) {
				return true;
			}
		}
		return false;
	}
}

